; 'delay' is a wrapper routine which uses the DEL1mS routine
; to generate a specified number of mS delay
; Enter with the number of mS to delay in TEMPH,TEMPL,
; from 1 to 65535mS (1mS => 65.535 Sec).
; Note the accuracy of the routine varries a bit, but the
; 1mS routine is fairly accurate.
; TEMPH and TEMPL are destroyed in the routine and two
; stack locations are used.
;
; REMEMBER -- both TEMPL and TEMPH are used even if you
; want a 2mS delay, you've got to CLR TEMPH


delay:	push	TEMPH		; save the delay values
	push	TEMPL

dl10:	pop	TEMPL		; check the lower byte value
	tst	TEMPL		; for zero
	breq	bumphi		; if so, go dec the high byte value
	dec	TEMPL		; else dec the low byte
	push	TEMPL		; save it
	rcall	DEL1mS		; and delay another mS
	rjmp	dl10

bumphi: rcall	DEL1mS		; do a delay
	dec	TEMPL		; note it
	pop	TEMPH		; check the upper byte value
	tst	TEMPH		; for zero
	breq	dedone		; if so, done, branch
	dec	TEMPH		; else decrement and loop back
	push	TEMPH
	push	TEMPL
	rjmp	dl10

dedone:	ret			; stack is cleared, ok to return

; DEL1mS is a a general purpose 1mS delay routine
; see comments for timing accuracy.

; Determine the delay values for the 1mS delay routine based upon
; the speed (in MHz) of the device used.  The speed of the device
; is set in the device.inc file.
               
.equ	T1val = 18*(Speed == 8) + 40*(Speed == 4) + 2*(Speed == 2) + 141*(Speed == 1)
.equ	T2val = 61*(Speed == 8) + 14*(Speed == 4) + 105*(Speed == 2) + 1*(Speed == 1)

DEL1mS:	ldi	TEMP2,T2val	;			  1 cycle
djmp00:	ldi	TEMP,T1val	;             1 cycle
djmp0:	rjmp	djmp1		; 2 cycles
djmp1:	rjmp	djmp2		; 2 cycles
djmp2:	dec	TEMP		; 1 cycle
	brne	djmp0		; 2 cycles or 1 cycle
	dec	TEMP2		;             1 cycle
	brne	djmp00		;             2 cycles or 1 cycle
	ret			; ---	     ---	 ---
				; 7	      5           2
				; inner      middle      outer
				; TEMP       TEMP2       once
				; 8000 x 125 nS = 1mS
				; 8000 = TEMP2 x (5 + 7 x TEMP1) + 2
				; rcall= 3 cycles, ret = 4 cycles
				; so... 8000=TEMP2x(5+7xTEMP1)+2+7
				; TEMP1 = 18, TEMP2 = 61, =>8000
				; 8000 x 125nS = 1.000000 mS
				;
				; at 4MHz
				; TEMP1=40, TEMP2=14
				; 3999 x 250nS = 0.999750 mS
				;
				; at 2MHz
				; TEMP1=2, TEMP2=105
				; 2004 x 500nS = 1.0020 mS
				;
				; at 1MHz
				; TEMP1=141, TEMP2=1
				; 1001 x 1000nS = 1.0010 mS
